@@ -7,7 +7,7 @@ from dagster.utils.backcompat import rename_warning
 
 
 class IntermediateStorageDefinition(IConfigMappable):
-    '''Defines intermediate data storage behaviors.
+    """Defines intermediate data storage behaviors.
     Args:
         name (str): Name of the storage mode.
         is_persistent (bool): Whether the storage is persistent in a way that can cross process/node
@@ -22,7 +22,7 @@ class IntermediateStorageDefinition(IConfigMappable):
             a :py:class:`IntermediateStorage`.
         _configured_config_mapping_fn: This argument is for internal use only. Users should not
             specify this field. To preconfigure a resource, use the :py:func:`configured` API.
-    '''
+    """
 
     def __init__(
         self,
@@ -33,26 +33,26 @@ class IntermediateStorageDefinition(IConfigMappable):
         intermediate_storage_creation_fn=None,
         _configured_config_mapping_fn=None,
     ):
-        self._name = check.str_param(name, 'name')
-        self._is_persistent = check.bool_param(is_persistent, 'is_persistent')
-        self._config_schema = check_user_facing_opt_config_param(config_schema, 'config_schema')
+        self._name = check.str_param(name, "name")
+        self._is_persistent = check.bool_param(is_persistent, "is_persistent")
+        self._config_schema = check_user_facing_opt_config_param(config_schema, "config_schema")
         self._intermediate_storage_creation_fn = check.opt_callable_param(
-            intermediate_storage_creation_fn, 'intermediate_storage_creation_fn'
+            intermediate_storage_creation_fn, "intermediate_storage_creation_fn"
         )
         self._required_resource_keys = frozenset(
             check.set_param(
                 required_resource_keys if required_resource_keys else set(),
-                'required_resource_keys',
+                "required_resource_keys",
                 of_type=str,
             )
         )
         self.__configured_config_mapping_fn = check.opt_callable_param(
-            _configured_config_mapping_fn, 'config_mapping_fn'
+            _configured_config_mapping_fn, "config_mapping_fn"
         )
 
     @property
     def config_field(self):
-        rename_warning('config_schema', 'config_field', '0.9.0')
+        rename_warning("config_schema", "config_field", "0.9.0")
         return self.config_schema
 
     @property
@@ -80,7 +80,7 @@ class IntermediateStorageDefinition(IConfigMappable):
         return self.__configured_config_mapping_fn
 
     def configured(self, config_or_config_fn, config_schema=None, **kwargs):
-        '''
+        """
         Wraps this object in an object of the same type that provides configuration to the inner
         object.
 
@@ -96,8 +96,8 @@ class IntermediateStorageDefinition(IConfigMappable):
                 of the storage mode being configured.
 
         Returns (IntermediateStorageDefinition): A configured version of this object.
-        '''
-        name = check.opt_str_param(kwargs.get('name'), 'name', self.name)
+        """
+        name = check.opt_str_param(kwargs.get("name"), "name", self.name)
         wrapped_config_mapping_fn = self._get_wrapped_config_mapping_fn(
             config_or_config_fn, config_schema
         )
@@ -115,7 +115,7 @@ class IntermediateStorageDefinition(IConfigMappable):
 def intermediate_storage(
     required_resource_keys=None, name=None, is_persistent=True, config_schema=None
 ):
-    '''Creates an intermediate storage definition
+    """Creates an intermediate storage definition
 
     The decorated function will be passed as the ``intermediate_storage_creation_fn`` to a
     :py:class:`IntermediateStorageDefinition`.
@@ -129,7 +129,7 @@ def intermediate_storage(
             The resources that this storage needs at runtime to function.
         config_schema (Optional[ConfigSchema]): The schema for the config. Configuration data available in
             `init_context.intermediate_storage_config`.
-    '''
+    """
 
     if callable(name):
         check.invariant(is_persistent is True)
@@ -149,15 +149,15 @@ class _IntermediateStorageDecoratorCallable(object):
     def __init__(
         self, name=None, is_persistent=True, config_schema=None, required_resource_keys=None
     ):
-        self.name = check.opt_str_param(name, 'name')
-        self.is_persistent = check.bool_param(is_persistent, 'is_persistent')
-        self.config_schema = check_user_facing_opt_config_param(config_schema, 'config_schema')
+        self.name = check.opt_str_param(name, "name")
+        self.is_persistent = check.bool_param(is_persistent, "is_persistent")
+        self.config_schema = check_user_facing_opt_config_param(config_schema, "config_schema")
         self.required_resource_keys = check.opt_set_param(
-            required_resource_keys, 'required_resource_keys', of_type=str
+            required_resource_keys, "required_resource_keys", of_type=str
         )
 
     def __call__(self, fn):
-        check.callable_param(fn, 'fn')
+        check.callable_param(fn, "fn")
 
         if not self.name:
             self.name = fn.__name__
